@@ -1,25 +1,25 @@
 // Nr. of milliseconds between execution of function mtoplink
 var btts=1000;
 
-// Location of 'back to page' from right lower corner of window
-bttx=130, btty=30;
-
 // What to display
 bttc='Back to top';
 
-// Determine window size en scroll location
-function winWid(){ return (ns4||ns6)?window.innerWidth:document.body.clientWidth; }
-function winHei(){ return (ns4||ns6)?window.innerHeight:document.body.clientHeight; }
-function winOfy(){ return (ns4||ns6)?window.pageYOffset:document.body.scrollTop; }
+// Determine browser
+var ns4=(document.layers)?1:0;
+var ie4=(document.all)?1:0;
+var ns6=(document.getElementById&&!document.all)?1:0;
+var hand=(ie4)?'hand':'pointer';
+if(ie4)	bttll="document.all['bttl'].style";
+if(ns4) bttll="document.layers['bttl']";
+if(ns6) bttll="document.getElementById('bttl').style";
 
-// Write link on page
-function mtoplink(){ 
-	with(eval(bttll)){ 
-		left=(winOfy()>0)?winWid()-bttx:-500; 
-		top=(winOfy()>0)?winHei()-btty+winOfy():-500; 
-	} 
-	setTimeout('mtoplink()',btts); 
-}
+// Location of 'back to page' from right lower corner of window
+// depends on browser how they handle the horizontal scrollbar...
+bttx=130;
+btty=30;
+if (ns4||ns6) btty=50;
+
+toplink();
 
 // Execute link
 function toplink(){ 
@@ -31,13 +31,18 @@ function toplink(){
 	document.write(rt); 
 	mtoplink(); 
 }
+// Write link on page
+function mtoplink(){ 
+	with(eval(bttll)){ 
+		left=(winOfy()>0)?winWid()-bttx:-500; 
+		top=(winOfy()>0)?winHei()-btty+winOfy():-500; 
+	} 
+	setTimeout('mtoplink()',btts); 
+}
+
+// Determine window size en scroll location
+function winWid(){ return (ns4||ns6)?window.innerWidth:document.body.clientWidth; }
+function winHei(){ return (ns4||ns6)?window.innerHeight:document.body.clientHeight; }
+function winOfy(){ return (ns4||ns6)?window.pageYOffset:document.body.scrollTop; }
 
-var ns4=(document.layers)?1:0;
-var ie4=(document.all)?1:0;
-var ns6=(document.getElementById&&!document.all)?1:0;
-var hand=(ie4)?'hand':'pointer';
-if(ie4)	bttll="document.all['bttl'].style";
-if(ns4) bttll="document.layers['bttl']";
-if(ns6) bttll="document.getElementById('bttl').style";
 
-toplink();
\ No newline at end of file
