@@ -16,12 +16,16 @@
 # ObjectUsed	Marks the object as used in the given rulebase
 # GetKey		Returns the value of a specific key
 # SetKey        Set the value of a key and returns the ref to key
+# CPVersion     Returns the version of CheckPoint installed if any
 ##########################################################################
 # This routine is not supported by Check Point in any way!
 # Use of this software is free, but is for your own risk!
 ##########################################################################
 package CheckPoint;
 
+my $SCR_NAME 	= 'CheckPoint';
+my $VERSION  	= '1.02';
+
 use strict;
 use Debug;
 use CPobject;
@@ -30,7 +34,7 @@ use Strings;
 our (@ISA, @EXPORT);
 require Exporter;
 @ISA = qw(Exporter Debug);
-@EXPORT = qw(GetObjects GetMembers ObjectUsed GetKey SetKey);
+@EXPORT = qw(GetObjects GetMembers ObjectUsed GetKey SetKey Version);
 
 # Global variables ...
 my $objects;
@@ -46,8 +50,10 @@ my $_CPclass = 'CPobject';
 #
 # Parameters:
 # $file   - Filename of CheckPoint databasefile
+# $table  - Name of table stored in the file
 #
 # Returns:
+# -1 if file can not be read (does not exist)
 # $objects - Root of objects tree
 sub GetObjects {
 
@@ -55,22 +61,34 @@ sub GetObjects {
 	my ($line, $level);
 	my ($section, $key);
 
-	my $file   = shift;
+	my $file 	= shift;
+	my $table	= shift;
 
-	if ($objects) {
-		# already initialized ...
-	} else {
-		# first table to be read, so initialise root object!
-		$objects = CPobject->new();
-	}
-
-	open (INFILE, "$file")
-		or die "Cannot open the CheckPoint database $file!\n\n";
-
-	$level = 0;
-	&ReadObject(undef, undef, $objects, $level);
-
-	close (INFILE);
+	if (-e "$file") {
+    	if ($objects) {
+    		# already initialized ...
+    	} else {
+    		# first table to be read, so initialise root object!
+    		$objects = CPobject->new();
+    	}
+    
+    	open (INFILE, "$file")
+    		or die "Cannot open the CheckPoint database $file!\n\n";
+    
+    	# Create table ...
+    	$level = 0;
+    	if ($table) {
+    		$objects->{$table} = {};
+    		&ReadObject($objects, $table, $objects->{$table}, $level);
+    	} else {
+    		&ReadObject(undef, undef, $objects, $level);
+    	}
+    
+    	close (INFILE);
+    } else {
+    	# File does not exist, return errorcode
+    	return -1;
+    }
 	return $objects;
 }
 
@@ -127,14 +145,17 @@ sub ReadObject {
 			if (((defined($key)) && ($key eq '')) &&
 				((defined($ref)) && ($ref eq ''))) { $ref='AnonymousObject' }
 
-
+			# Clear fields with only whitespaces (only $val can be empty string) ...
+			if ($key !~ /\S+/) { undef $key }
+			if ($ref !~ /\S+/) { undef $ref }
+			
 			# Found a VALUE to be set
 			# ':key (value)'
 			# ': (value)'
 			if (defined($val)) {
 				&debug("[$level]"." "x$level.":".$key." (".$val.")\n") if ($key ne '');
 				&debug("[$level]"." "x$level.": (".$val.")\n")         if ($key eq '');
-				if ($key) {
+				if (defined($key)) {
 					$current->{$key} = $val;
 				} else {
 					# Special case: $parent checks out to be a list of values,
@@ -150,7 +171,7 @@ sub ReadObject {
 			# Found a KEY of the existing object
 			# ':key ('
 			# ':key (ref' -> ref is used for comments only. ignored
-			} elsif ($key ne '') {
+			} elsif (defined($key)) {
 				&debug("[$level]"." "x$level.":".$key." ("."\n") 		if (!$ref);
 				&debug("[$level]"." "x$level.":".$key." (".$ref."\n") 	if ($ref);
 
@@ -180,7 +201,7 @@ sub ReadObject {
 
 			# Found a reference to an OBJECT called REF
 			# ': (ref'
-			} elsif ($ref ne '') {
+			} elsif (defined($ref)) {
 				&debug("[$level]"." "x$level.": (".$ref."\n");
 				# Create new CPobject for REF
 				$child = CPobject->new();
@@ -243,7 +264,9 @@ sub GetKey {
 	while ($keystr ne '') {
 		@key = split(/:/, $keystr);
 		foreach $key (@key) {
-			if ((ref($obj) ne '') && exists($obj->{$key})) {
+			if (ref($obj) eq 'ARRAY') {
+				return "";
+			} elsif ((ref($obj) ne '') && exists($obj->{$key})) {
 				$obj=$obj->{$key}
 			} else {
 				return "";
@@ -366,7 +389,7 @@ sub GetMembers {
 		@members=();
 	} elsif (ref($hash) eq 'HASH') {
 		# This is a weird case, return reference to itself;
-		# Might be a a list of one object, and is therefore note converted
+		# Might be a a list of one object, and is therefore not converted
 		# to a list yet. F.i. a single rule in a rulebase,
 		# or an action rulebase object
 		push @members, $hash
@@ -396,7 +419,7 @@ sub ObjectUsed {
 	my ($is_used_before);
 
 	$object = shift;
-	if ($object eq __PACKAGE__) {$object = shift; print "*** object shifted\n"}
+	if ($object eq __PACKAGE__) { $object = shift }
 	$key = shift;
 	$n = shift;
 
@@ -454,12 +477,40 @@ sub ObjectUsed {
         		}
         	}
     	} else {
-    		print "To many recursions found, skipping ...\n";
+    		print "Too many recursions found, skipping ...\n";
     	}
     }
 	return $nrobj;
 }
 
+##########################################################################
+# Routine: Version
+#
+# Description:
+# This function returns the version of a CheckPoint object. If no
+# CheckPoint software is installed, it returns an empty string.
+#
+# Parameters:
+# $object  Object/key to flag used
+#
+# Returns:
+# $version  The version of the object
+sub Version {
+	use strict;
+	my ($object, $version);
+
+	$object = shift;
+	if ($object eq __PACKAGE__) { $object = shift }
+
+
+	# Determine version of CheckPoint software installed ...
+	if (GetKey($object, 'cp_products_installed') eq 'true') {
+		$version = (GetKey($object, 'cpver')||GetKey($object, 'fwver')).GetKey($object, 'option_pack');
+	}
+	
+	return $version;
+}
+
 1;
 
 __END__
@@ -547,7 +598,7 @@ B<Note:> All the data is retained within the package and can only be accessed th
 
 =head2 GetObjects
 
-The first step of any program would be to read the CheckPoint database files in to memory. The most important file is currently 'objects_5_0.C' which holds all objects and its parameters. Other files are 'rulebases_5_0.fws' and 'user.C' (although it is not clear to me what this file exactly is meant to do).
+The first step of any program would be to read the CheckPoint database files in to memory. The most important file is currently 'objects_5_0.C' which holds all objects and its parameters. Other files are 'rulebases_5_0.fws' and 'users.C' (although it is not clear to me what this file exactly is meant to do).
 
 B<Syntax:> GetObjects(I<filename>)
 
@@ -711,6 +762,24 @@ B<Examples:>
   $NrObjectsChanged = ObjectUsed(GetKey('network_objects:MyIntranet'), 'ThisRulebase');
 
 
+=head2 CPVersion
+
+The routine I<CPVersion> is  used to get the version of the Check Point software installed on the object. If any. 
+
+B<Returns:> The version of the object, starting with the main version number, a dot, the minor version number, followed by the option pack number (feature pack). Possible values observed at this time(!):
+
+  4.14  Check Point version 4.1 Option Pack 4
+  5.00  Check Point Next Generation
+  5.01  Check Point Next Generation Feature Pack 1
+  5.02  Check Point Next Generation Feature Pack 2
+  5.03  Check Point Next Generation Feature Pack 3
+  5.04  Check Point Next Generation with Application Intelligence
+
+B<Examples:>
+
+  $Version = CPVersion(GetKey('network_objects:MyGateway'));
+  
+
 =head1 History
 
 This is version 1.0. There is no history yet, as this is the first release...
